;********************************************************
;
;                 Temperature controller
;
;                                 Device : PIC16F873
;                                 Clock  : 4MHz
;                                 Author : Seiichi Inoue
;********************************************************

        processor       16F873
        include         p16f873.inc
        radix           dec
        __config        h'3F32' ;OSC is HS
                                ;RB3,RB6,RB7 are I/O
                                ;Brown-out detection OFF
                                ;Power-up timer ON
                                ;Code protection OFF
                                ;Data code protection OFF
                                ;Watchdog timer OFF
                                ;LVP OFF
        errorlevel      -302    ;Suppress bank warning

;****************  Label Definition  ********************
        cblock  h'20'
out10                           ;10th of out-side(BCD)
out1                            ;1st of out-side(BCD)
in10                            ;10th of in-side(BCD)
in1                             ;1st of in-side(BCD)
preset10                        ;10th of preset(BCD)
preset1                         ;1st of preset(BCD)
out_temp1                       ;Out-side(8 bits)
in_temp1                        ;In-side(8 bits)
out_temp2                       ;Out-side(6 bits)
in_temp2                        ;In-side(6 bits)
preset2                         ;preset(6 bits)
interval                        ;Interval sw(BCD)
mode                            ;Mode(1:up 0:down)
test_mode                       ;Test(2:OFF,1:IN,0:OUT)
scan_cont                       ;Scanning conter
rc_cont                         ;RC control data
rb_data                         ;RB data
t25us_cnt                       ;25 microseconds counter
t200us_cnt                      ;200 microseconds counter
t1ms_cnt                        ;1 millisecond counter

seg_ha                          ;7seg LED table head adr
seg0                            ;0
seg1                            ;1
seg2                            ;2
seg3                            ;3
seg4                            ;4
seg5                            ;5
seg6                            ;6
seg7                            ;7
seg8                            ;8
seg9                            ;9
sega                            ;A(Blank)

w_save                          ;W reg save area
s_save                          ;STATUS reg save area
temp_check                      ;Temperature check(2S)
cont_flag                       ;Control flag
presetb                         ;Binary convert conter
repeat                          ;Preset repeat conter
presetw                         ;Preset work area
presetlk                        ;Preset last look
interval_h                      ;Higher interval timer
interval_l                      ;Lower interval timer
        endc

flag    equ     0               ;Control flag position
o1      equ     2               ;Output1 control position
o2      equ     5               ;Output2 control position

;********************************************************
;                    Program Start
;********************************************************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;------------------ Initial Process ---------------------
init

;*** Port initialization
        bsf     status,rp0      ;Change to Bank1
        movlw   b'00011011'     ;AN0/1/3/RA4:IN RA2/5:OUT
        movwf   trisa           ;Set RA port mode
        clrf    trisb           ;Set RB port output mode
        clrf    trisc           ;Set RC port output mode

;*** Timer 0 initialization (for temp detection : 10ms)
        movlw   b'00000101'     ;PS=1:64
        movwf   option_reg      ;Set OPTION_REG register
        bcf     status,rp0      ;Change to Bank0
        movlw   100             ;256-10000us/64us=100
        movwf   tmr0            ;Set TMR0

;*** Timer 1 initialization (for hold timer : 500ms)
        movlw   b'00110001'     ;Pre=1:8 TMR1=Int TMR1=ON
        movwf   t1con           ;Set T1CON register
        movlw   h'0b'           ;65536-500000us/8us=3036
        movwf   tmr1h           ;Set TMR1H register
        movlw   h'dc'           ;3036=h'BDC'
        movwf   tmr1l           ;Set TMR1L register
        bsf     status,rp0      ;Change to Bank1
        bsf     pie1,0          ;Set TMR1IE

;*** A/D converter initialization
        movlw   b'00000100'     ;ADFM=0 PCFG=0100
        movwf   adcon1          ;Set ADCON1 register
        bcf     status,rp0      ;Change to Bank0

;*** 7seg LED pattern data setting
        movlw   seg0            ;Set table head address
        movwf   seg_ha
        movlw   b'11000000'     ;0
        movwf   seg0
        movlw   b'11111001'     ;1
        movwf   seg1
        movlw   b'10100100'     ;2
        movwf   seg2
        movlw   b'10110000'     ;3
        movwf   seg3
        movlw   b'10011001'     ;4
        movwf   seg4
        movlw   b'10010010'     ;5
        movwf   seg5
        movlw   b'10000010'     ;6
        movwf   seg6
        movlw   b'11111000'     ;7
        movwf   seg7
        movlw   b'10000000'     ;8
        movwf   seg8
        movlw   b'10010000'     ;9
        movwf   seg9
        movlw   b'11111111'     ;A(Blank)
        movwf   sega

;*** Work area initialization
        clrf    porta           ;Clear PORTA
        movlw   b'11111111'
        movwf   portb           ;Set PORTB
        movwf   portc           ;Set PORTC
        movlw   9
        movwf   scan_cont       ;Set scanning counter
        movlw   h'a'            ;Display turning-off
        movwf   out10
        movwf   out1
        movwf   in10
        movwf   in1
        movwf   preset10
        movwf   preset1
        clrf    preset2
        clrf    repeat
        movlw   200
        movwf   temp_check      ;Temp check interval(2S)
        clrf    cont_flag       ;Clear control flag

;*** Interruption control
        movlw   b'11100000'     ;GIE=ON,PEIE=ON,T0IE=ON
        movwf   intcon          ;Set INTCON register

;------------- LED and SW scanning Process --------------
scan
        movfw   scan_cont       ;Read scanning counter
        sublw   9
        btfsc   status,z        ;9 ?
        goto    process1        ;10th of out-side disp
        movfw   scan_cont       ;Read scanning counter
        sublw   8
        btfsc   status,z        ;8 ?
        goto    process2        ;1st of out-side disp
        movfw   scan_cont       ;Read scanning counter
        sublw   7
        btfsc   status,z        ;7 ?
        goto    process3        ;10th of in-side disp
        movfw   scan_cont       ;Read scanning counter
        sublw   6
        btfsc   status,z        ;6 ?
        goto    process4        ;1st of in-side dsip
        movfw   scan_cont       ;Read scanning counter
        sublw   5
        btfsc   status,z        ;5 ?
        goto    process5        ;10th of preset disp
        movfw   scan_cont       ;Read scanning counter
        sublw   4
        btfsc   status,z        ;4 ?
        goto    process6        ;1st of preset disp
        movfw   scan_cont       ;Read scanning counter
        sublw   3
        btfsc   status,z        ;3 ?
        goto    process7        ;10th of preset sw
        movfw   scan_cont       ;Read scanning counter
        sublw   2
        btfsc   status,z        ;2 ?
        goto    process8        ;1st of preset sw
        movfw   scan_cont       ;Read scanning counter
        sublw   1
        btfsc   status,z        ;1 ?
        goto    process9        ;Interval sw

scan1
        movlw   9
        movwf   scan_cont       ;Set scanning counter
        goto    scan

process1
        movlw   b'11111110'
        movwf   rc_cont         ;Set RC control data
        movfw   out10           ;Read 10th of out-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process2
        movlw   b'11111101'
        movwf   rc_cont         ;Set RC control data
        movfw   out1            ;Read 1st of out-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process3
        movlw   b'11111011'
        movwf   rc_cont         ;Set RC control data
        movfw   in10            ;Read 10th of in-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process4
        movlw   b'11110111'
        movwf   rc_cont         ;Set RC control data
        movfw   in1             ;Read 1st of in-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process5
        movlw   b'11101111'
        movwf   rc_cont         ;Set RC control data
        movfw   preset10        ;Read 10th of preset
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process6
        movlw   b'11011111'
        movwf   rc_cont         ;Set RC control data
        movfw   preset1         ;Read 1st of preset
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process7
        movlw   b'11111111'
        movwf   portb           ;LED OFF
        bsf     status,rp0      ;Change to Bank1
        movlw   b'01111111'     ;RB0-6:IN RB7:OUT
        movwf   trisb           ;Set RB port mode
        bcf     status,rp0      ;Change to Bank0
        movlw   b'10111111'
        movwf   portc           ;Set PORTC
        call    t200us          ;Wait 200 microseconds
        movfw   portb           ;Read PORTB
        xorlw   b'11111111'     ;Reverse bits
        movwf   preset10        ;Save 10th of preset
        movlw   b'00001111'     ;Set mask
        andwf   preset10,f      ;Pick up sw data
        goto    proc_end

process8
        movlw   b'01111111'
        movwf   portc           ;Set PORTC
        call    t200us          ;Wait 200 microseconds
        movfw   portb           ;Read PORTB
        xorlw   b'11111111'     ;Reverse bits
        movwf   preset1         ;Save 1st of preset
        movlw   b'00001111'     ;Set mask
        andwf   preset1,w       ;Pick up sw data
        movwf   preset1         ;Save 1st of preset
        movwf   presetw         ;Save preset(Binary)
        movfw   preset10        ;Read 10th of preset
        incf    preset10,w      ;10th of preset + 1
        movwf   presetb         ;Save count
process81
        decfsz  presetb,f       ;Count - 1 = 0 ?
        goto    process84       ;No.
        movfw   repeat          ;Yes. Read repeat counter
        btfss   status,z        ;Repeat counter = 0 ?
        goto    process82       ;No.
        movfw   presetw         ;Yes. Read preset data
        movwf   presetlk        ;Save preset last look
        incf    repeat,f        ;Repeat conter + 1
        goto    proc_end
process82
        movfw   presetlk        ;Read last look
        subwf   presetw,w       ;Preser - last look
        btfss   status,z        ;Preset = last look ?
        goto    precess83       ;No.
        incf    repeat,f        ;Yes. Repeat conter + 1
        movlw   3               ;Set reference data
        subwf   repeat,w        ;Repeat counter - 3
        btfss   status,z        ;Repeat counter = 3 ?
        goto    proc_end        ;No.
        movfw   presetw         ;Read preset data
        movwf   preset2         ;Set preset data
precess83
        clrf    repeat          ;Clear repeat conter
        goto    proc_end

process84
        movlw   10              ;Set add value
        addwf   presetw,f       ;preset + 10
        goto    process81

process9
        movlw   b'11111111'
        movwf   portc           ;Set PORTC
        bcf     portb,7         ;Set PORTB
        call    t200us          ;Wait 200 microseconds
        movfw   portb           ;Read PORTB
        xorlw   b'11111111'     ;Reverse bits
        movwf   interval        ;Save interval data
        movlw   b'00001111'     ;Set mask
        andwf   interval,f      ;Pick up sw data
        movfw   interval        ;Read interval data
        btfss   status,z        ;Interval data = 0 ?
        goto    process91
        movlw   10              ;Yes. Set 10
        movwf   interval        ;Save interval data
process91
        bsf     status,rp0      ;Change to Bank1
        clrf    trisb           ;Set RB port output mode
        bcf     status,rp0      ;Change to Bank0
        movlw   b'11111111'
        movwf   portb           ;LED OFF

;*** UP/DOWN mode check
        btfss   porta,4         ;RA4 = 1 ?
        goto    proc91          ;No.
        movlw   1
        movwf   mode            ;Set UP mode
        goto    proc_end
proc91
        clrf    mode            ;Set DOWN mode

proc_end
        decfsz  scan_cont,f     ;Scan end ?
        goto    scan            ;No.
        goto    scan1           ;Yes

;--------------------------------------------------------
;                  LED ON subroutine
;--------------------------------------------------------
led_on
        movlw   b'11111111'
        movwf   portb           ;LED OFF
        movfw   rc_cont         ;Read RC control data
        movwf   portc           ;Set PORTC
        call    t200us          ;Wait 200 microseconds
        movfw   rb_data         ;Read disp digit
        addwf   seg_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        call    t1ms            ;Wait 1 millisecond
        return

;--------------------------------------------------------
;              200 microseconds subroutine
;--------------------------------------------------------
t200us
        movlw   48
        movwf   t200us_cnt      ;Set loop counter
t200us_loop
        nop
        decfsz  t200us_cnt,f    ;Counter - 1
        goto    t200us_loop     ;Not end
        return

;--------------------------------------------------------
;               1 millisecond subroutine
;--------------------------------------------------------
t1ms
        movlw   5
        movwf   t1ms_cnt        ;Set loop counter
t1ms_loop
        call    t200us
        decfsz  t1ms_cnt,f      ;Counter - 1
        goto    t1ms_loop       ;Not end
        return

;********************************************************
;                 Interruption Process
;********************************************************
int
        movwf   w_save          ;Save W register
        movfw   status          ;Read STATUS reg
        movwf   s_save          ;Save STATUS reg
        bcf     status,rp0      ;Change to Bank0
        btfsc   intcon,t0if     ;Check TMR0
        goto    temp_detect     ;TMR0 time-out
        btfsc   pir1,tmr1if     ;Check TMR1IF
        goto    out_control     ;TMR1 time-out

;********************************************************
;              END of Interruption Process
;********************************************************
int_end
        movfw   s_save          ;Read saved STATUS reg
        movwf   status          ;Recover STATUS reg
        swapf   w_save,f        ;Read saved W register
        swapf   w_save,w        ;Recover W register
        retfie

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;         Temperature detection Process (10ms)
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
temp_detect
        bcf     intcon,t0if     ;Clear T0IF
        movlw   100             ;Set Time value
        movwf   tmr0            ;Write TMR0 register
        decfsz  temp_check,f    ;count - 1
        goto    int_end
        movlw   200
        movwf   temp_check      ;Set 2 seconds

;------------------ TEST mode check ---------------------
mode1
        movlw   b'01011001'     ;ADCS=01 CHS=AN3 ADON=ON
        movwf   adcon0          ;Set ADCON0 register
        call    t25us           ;Wait 25 microseconds
        bsf     adcon0,2        ;Start A/D conversion
mode2
        btfsc   adcon0,2        ;Conversion end ?
        goto    mode2           ;No.
        movlw   168             ;Set 3.3V
        subwf   adresh,w        ;AN3 - 3.3V
        btfss   status,c        ;AN3 >= 3.3V
        goto    mode3           ;No.
        movlw   2               ;Yes.
        movwf   test_mode       ;Set TEST OFF
        goto    temp0
mode3
        movlw   87              ;Set 1.7V
        subwf   adresh,w        ;AN3 - 1.7V
        btfss   status,c        ;AN3 >= 1.7V
        goto    mode4           ;No.
        movlw   1               ;Yes.
        movwf   test_mode       ;Set IN TEST
        goto    temp0
mode4
        clrf    test_mode       ;Set OUT TEST

;------------ OUT-SIDE temperature detection ------------
temp0
        movlw   b'01001001'     ;ADCS=01 CHS=AN1 ADON=ON
        movwf   adcon0          ;Set ADCON0 register
        call    t25us           ;Wait 25 microseconds
        bsf     adcon0,2        ;Start A/D conversion
temp1
        btfsc   adcon0,2        ;Conversion end ?
        goto    temp1           ;No.
        movfw   adresh          ;Read result
        addlw   8               ;Add 2deg-c
        movwf   out_temp1       ;Save (unit:0.25deg-c)
        movwf   out_temp2       ;Set data
        rrf     out_temp2,f     ;Rotate Right
        rrf     out_temp2,f     ;Rotate Right
        movlw   b'00111111'     ;Set mask
        andwf   out_temp2,f     ;Save (unit:1deg-c)

;------------ IN-SIDE temperature detection -------------
        movlw   b'01000001'     ;ADCS=01 CHS=AN0 ADON=ON
        movwf   adcon0          ;Set ADCON0 register
        call    t25us           ;Wait 25 microseconds
        bsf     adcon0,2        ;Start A/D conversion
temp2
        btfsc   adcon0,2        ;Conversion end ?
        goto    temp2           ;No.
        movfw   adresh          ;Read result
        addlw   8               ;Add 2deg-c
        movwf   in_temp1        ;Save (unit:0.25deg-c)
        movwf   in_temp2        ;Set data
        rrf     in_temp2,f      ;Rotate Right
        rrf     in_temp2,f      ;Rotate Right
        movlw   b'00111111'     ;Set mask
        andwf   in_temp2,f      ;Save (unit:1deg-c)

;--------------- Display data conversion ----------------
        movfw   test_mode       ;Read test mode
        sublw   2               ;Set check data
        btfsc   status,z        ;Check
        goto    temp30          ;2(TEST OFF)
        movfw   test_mode       ;Read test mode
        sublw   1               ;Set check data
        btfsc   status,z        ;Check
        goto    temp40          ;1(IN-SIDE DATA)
        goto    temp50          ;0(OUT-SIDE DATA)

;*** TEST OFF
;(OUT-SIDE)
temp30
        clrf    out10           ;Clear 10th of out-side
        movfw   out_temp2       ;Read out-side 1deg data
        movwf   out1            ;Set data
        movlw   10              ;Set divisor
temp31
        subwf   out1,f          ;Temp - 10
        btfss   status,c        ;Result >= 0 ?
        goto    temp32          ;No. Jump to correction
        incf    out10,f         ;Yes. 10th of out-side + 1
        goto    temp31          ;Continue
temp32
        movlw   10              ;Correction value
        addwf   out1,f          ;Correction
;(IN-SIDE)
temp33
        clrf    in10            ;Clear 10th of in-side
        movfw   in_temp2        ;Read in-side 1deg data
        movwf   in1             ;Set data
        movlw   10              ;Set divisor
temp34
        subwf   in1,f           ;Temp - 10
        btfss   status,c        ;Result >= 0 ?
        goto    temp35          ;No. Jump to correction
        incf    in10,f          ;Yes. 10th of out-side + 1
        goto    temp34          ;Continue
temp35
        movlw   10              ;Correction value
        addwf   in1,f           ;Correction
        goto    comp0           ;Jump to comparison

;*** IN-SIDE DATA display
temp40
        clrf    out10           ;Clear 10th of out-side
        movfw   in_temp2        ;Read in-side 1deg data
        movwf   out1            ;Set data
        movlw   10              ;Set divisor
temp41
        subwf   out1,f          ;Temp - 10
        btfss   status,c        ;Result >= 0 ?
        goto    temp42          ;No. Jump to correction
        incf    out10,f         ;Yes. 10th of out-side + 1
        goto    temp41          ;Continue
temp42
        movlw   10              ;Correction value
        addwf   out1,f          ;Correction
temp43
        btfss   in_temp1,1      ;1X ?
        goto    temp45          ;No. 0X
        btfss   in_temp1,0      ;11 ?
        goto    temp44          ;No. 10
        movlw   7               ;Set .75 deg
        movwf   in10
        movlw   5
        movwf   in1
        goto    comp0           ;Jump to comparison
temp44
        movlw   5               ;Set .50 deg
        movwf   in10
        movlw   0
        movwf   in1
        goto    comp0           ;Jump to comparison
temp45
        btfss   in_temp1,0      ;01 ?
        goto    temp46          ;No. 00
        movlw   2               ;Set .25 deg
        movwf   in10
        movlw   5
        movwf   in1
        goto    comp0           ;Jump to comparison
temp46
        movlw   0               ;Set .00 deg
        movwf   in10
        movlw   0
        movwf   in1
        goto    comp0           ;Jump to comparison

;*** OUT-SIDE DATA display
temp50
        clrf    out10           ;Clear 10th of out-side
        movfw   out_temp2       ;Read out-side 1deg data
        movwf   out1            ;Set data
        movlw   10              ;Set divisor
temp51
        subwf   out1,f          ;Temp - 10
        btfss   status,c        ;Result >= 0 ?
        goto    temp52          ;No. Jump to correction
        incf    out10,f         ;Yes. 10th of out-side + 1
        goto    temp51          ;Continue
temp52
        movlw   10              ;Correction value
        addwf   out1,f          ;Correction
temp53
        btfss   out_temp1,1     ;1X ?
        goto    temp55          ;No. 0X
        btfss   out_temp1,0     ;11 ?
        goto    temp54          ;No. 10
        movlw   7               ;Set .75 deg
        movwf   in10
        movlw   5
        movwf   in1
        goto    comp0           ;Jump to comparison
temp54
        movlw   5               ;Set .50 deg
        movwf   in10
        movlw   0
        movwf   in1
        goto    comp0           ;Jump to comparison
temp55
        btfss   out_temp1,0     ;01 ?
        goto    temp56          ;No. 00
        movlw   2               ;Yes. Set .25 deg
        movwf   in10
        movlw   5
        movwf   in1
        goto    comp0           ;Jump to comparison
temp56
        movlw   0               ;Set .00 deg
        movwf   in10
        movlw   0
        movwf   in1
        goto    comp0           ;Jump to comparison

;---------- Temperature comparison processing -----------
comp0
        movfw   mode            ;Read mode(1:up 0:down)
        btfsc   status,z        ;Up ?
        goto    comp2           ;Jump to down mode

;***(Up mode)
        movfw   in_temp2        ;Read in-side
        subwf   preset2,w       ;preset - in-side
        btfsc   status,c        ;preset >= in-side ?
        goto    comp1           ;(>=)
        movfw   in_temp2        ;(<)Read in-side
        subwf   out_temp2,w     ;out-side - in-side
        btfsc   status,c        ;out-side >= in-side ?
        goto    comp1           ;(>=)
        bsf     cont_flag,flag  ;(<)Set control flag
        goto    int_end         ;Jump to int end
comp1
        bcf     cont_flag,flag  ;Clear control flag
        goto    int_end         ;Jump to int end

;***(Down mode)
comp2
        movfw   preset2         ;Read preset
        subwf   in_temp2,w      ;in-side - preset
        btfsc   status,c        ;in-side >= preset ?
        goto    comp1           ;(>=)
        bsf     cont_flag,flag  ;(<)Set control flag
        goto    int_end         ;Jump to int end

;--------------------------------------------------------
;              25 microseconds subroutine
;--------------------------------------------------------
t25us
        movlw   5
        movwf   t25us_cnt       ;Set loop counter
t25us_loop
        nop
        decfsz  t25us_cnt,f     ;Counter - 1
        goto    t25us_loop      ;Not end
        return

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;           Output control Process (500ms)
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
out_control
        bcf     pir1,tmr1if     ;Clear TMR1IF
        movlw   h'0b'           ;
        movwf   tmr1h           ;Set TMR1H register
        movlw   h'dc'           ;
        movwf   tmr1l           ;Set TMR1L register
        movfw   cont_flag       ;Read all control flags
        btfsc   status,z        ;All flags OFF ?
        goto    int_end         ;Yes. Jump to int end

        btfsc   cont_flag,o1    ;OUT1 control flag ON ?
        goto    cont1           ;Yes.
        btfsc   cont_flag,o2    ;OUT2 control flag ON ?
        goto    cont1           ;Yes.
;***(1st control)
        bsf     cont_flag,o1    ;Set OUT1 control flag
        bsf     porta,o1        ;OUT1 ON
        call    time_set        ;Set timer value
        goto    int_end         ;Jump to int end

cont1
        decfsz  interval_l,f    ;time -1
        goto    int_end         ;Not time-out
        movlw   1
        subwf   interval_h,f    ;Higher value check
        btfss   status,c        ;Value >= 1 ?
        goto    cont2           ;Value = 0 (Time-out)
        movlw   h'ff'
        movwf   interval_l      ;Set Lower value
        goto    int_end         ;Not time-out
cont2
        btfss   cont_flag,flag  ;Control flag ON ?
        goto    cont3           ;No.
        bsf     cont_flag,o2    ;Set output2 flag
        bsf     porta,o2        ;OUT2 ON
        call    time_set        ;Set timer value
        goto    int_end         ;Jump to int end
cont3
        btfss   cont_flag,o2    ;Output2 flag ON ?
        goto    cont4           ;No.
        bcf     cont_flag,o2    ;Clear output2 flag
        bcf     porta,o2        ;OUT2 OFF
        call    time_set        ;Set timer value
        goto    int_end         ;Jump to int end
cont4
        bcf     cont_flag,o1    ;Clear output1 flag
        bcf     porta,o1        ;OUT1 OFF
        goto    int_end         ;Jump to int end

;--------------------------------------------------------
;           Time-out value setting subroutine
;--------------------------------------------------------
time_set
        movfw   interval        ;Read interval sw value
        sublw   1
        btfsc   status,z        ;1 ?
        goto    set1            ;Set value fou 1 minute
        movfw   interval        ;Read interval sw value
        sublw   2
        btfsc   status,z        ;2 ?
        goto    set2            ;Set value fou 2 minutes
        movfw   interval        ;Read interval sw value
        sublw   3
        btfsc   status,z        ;3 ?
        goto    set3            ;Set value fou 3 minutes
        movfw   interval        ;Read interval sw value
        sublw   4
        btfsc   status,z        ;4 ?
        goto    set4            ;Set value fou 4 minutes
        movfw   interval        ;Read interval sw value
        sublw   5
        btfsc   status,z        ;5 ?
        goto    set5            ;Set value fou 5 minutes
        movfw   interval        ;Read interval sw value
        sublw   6
        btfsc   status,z        ;6 ?
        goto    set6            ;Set value fou 6 minutes
        movfw   interval        ;Read interval sw value
        sublw   7
        btfsc   status,z        ;7 ?
        goto    set7            ;Set value fou 7 minutes
        movfw   interval        ;Read interval sw value
        sublw   8
        btfsc   status,z        ;8 ?
        goto    set8            ;Set value fou 8 minutes
        movfw   interval        ;Read interval sw value
        sublw   9
        btfsc   status,z        ;9 ?
        goto    set9            ;Set value fou 9 minutes
        goto    set10           ;Set value fou 10 minutes

set1
        movlw   h'0'            ;Set 1 minute
        movwf   interval_h
        movlw   h'78'
        movwf   interval_l
        return
set2
        movlw   h'0'            ;Set 2 minutes
        movwf   interval_h
        movlw   h'f0'
        movwf   interval_l
        return
set3
        movlw   h'1'            ;Set 3 minutes
        movwf   interval_h
        movlw   h'68'
        movwf   interval_l
        return
set4
        movlw   h'1'            ;Set 4 minutes
        movwf   interval_h
        movlw   h'e0'
        movwf   interval_l
        return
set5
        movlw   h'2'            ;Set 5 minutes
        movwf   interval_h
        movlw   h'58'
        movwf   interval_l
        return
set6
        movlw   h'2'            ;Set 6 minutes
        movwf   interval_h
        movlw   h'd0'
        movwf   interval_l
        return
set7
        movlw   h'3'            ;Set 7 minutes
        movwf   interval_h
        movlw   h'48'
        movwf   interval_l
        return
set8
        movlw   h'3'            ;Set 8 minutes
        movwf   interval_h
        movlw   h'c0'
        movwf   interval_l
        return
set9
        movlw   h'4'            ;Set 9 minutes
        movwf   interval_h
        movlw   h'38'
        movwf   interval_l
        return
set10
        movlw   h'4'            ;Set 10 minutes
        movwf   interval_h
        movlw   h'b0'
        movwf   interval_l
        return

;********************************************************
;            END of Temperature controller
;********************************************************
        end
